/*!
 * template_js 3.1.3-alpha.0 (https://github.com/yanhaijing/template.js)
 * API https://github.com/yanhaijing/template.js/blob/master/README.md
 * Copyright 2017-2023 yanhaijing. All Rights Reserved
 * Licensed under MIT (https://github.com/yanhaijing/template.js/blob/master/LICENSE)
 */

(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, global.template = factory());
})(this, (function () { 'use strict';

  /*!
   * @jsmini/type 0.11.0 (https://github.com/jsmini/type)
   * API https://github.com/jsmini/type/blob/master/doc/api.md
   * Copyright 2017-2023 jsmini. All Rights Reserved
   * Licensed under MIT (https://github.com/jsmini/type/blob/master/LICENSE)
   */

  function _typeof(o) {
    "@babel/helpers - typeof";

    return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) {
      return typeof o;
    } : function (o) {
      return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o;
    }, _typeof(o);
  }
  var toString = Object.prototype.toString;
  function type(x) {
    var strict = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
    strict = !!strict;

    // fix typeof null = object
    if (x === null) {
      return 'null';
    }
    var t = _typeof(x);

    // 严格模式 区分NaN和number
    if (strict && t === 'number' && isNaN(x)) {
      return 'nan';
    }

    // number string boolean undefined symbol
    if (t !== 'object') {
      return t;
    }
    var cls;
    var clsLow;
    try {
      cls = toString.call(x).slice(8, -1);
      clsLow = cls.toLowerCase();
    } catch (e) {
      // ie下的 activex对象
      return 'object';
    }
    if (clsLow !== 'object') {
      if (strict) {
        // 区分NaN和new Number
        if (clsLow === 'number' && isNaN(x)) {
          return 'NaN';
        }
        // 区分 String() 和 new String()
        if (clsLow === 'number' || clsLow === 'boolean' || clsLow === 'string') {
          return cls;
        }
      }
      return clsLow;
    }

    // eslint-disable-next-line eqeqeq
    if (x.constructor == Object) {
      return clsLow;
    }

    // Object.create(null)
    try {
      // __proto__ 部分早期firefox浏览器
      if (Object.getPrototypeOf(x) === null || x.__proto__ === null) {
        return 'object';
      }
    } catch (e) {
      // ie下无Object.getPrototypeOf会报错
    }

    // function A() {}; new A
    try {
      var cname = x.constructor.name;
      if (typeof cname === 'string') {
        return cname;
      }
    } catch (e) {
      // 无constructor
    }

    // function A() {}; A.prototype.constructor = null; new A
    return 'unknown';
  }

  /*!
   * @jsmini/is 0.10.0 (https://github.com/jsmini/is)
   * API https://github.com/jsmini/is/blob/master/doc/api.md
   * Copyright 2017-2023 jsmini. All Rights Reserved
   * Licensed under MIT (https://github.com/jsmini/is/blob/master/LICENSE)
   */

  function isObject(x) {
    return type(x) === 'object';
  }
  function isFunction(x) {
    return type(x) === 'function';
  }
  var isArray = isFunction(Array.isArray) ? Array.isArray : function isArray(x) {
    return type(x) === 'array';
  };

  /*!
   * @jsmini/extend 0.5.0 (https://github.com/jsmini/extend)
   * API https://github.com/jsmini/extend/blob/master/doc/api.md
   * Copyright 2017-2023 jsmini. All Rights Reserved
   * Licensed under MIT (https://github.com/jsmini/extend/blob/master/LICENSE)
   */


  // Object.create(null) 的对象，没有hasOwnProperty方法
  function hasOwnProp(obj, key) {
    return Object.prototype.hasOwnProperty.call(obj, key);
  }
  var assign = isFunction(Object.assign) ? Object.assign : function assign(target) {
    if (!isObject(target)) {
      throw new TypeError('assign first param must is object');
    }
    for (var i = 0; i < (arguments.length <= 1 ? 0 : arguments.length - 1); i++) {
      var source = i + 1 < 1 || arguments.length <= i + 1 ? undefined : arguments[i + 1];
      if (isObject(source)) {
        for (var key in source) {
          if (hasOwnProp(source, key)) {
            target[key] = source[key];
          }
        }
      }
    }
    return target;
  };
  function extend() {
    return assign.apply(void 0, arguments);
  }
  function extendDeep(target) {
    // 深拷贝
    if (!isObject(target) && !isArray(target)) {
      throw new TypeError('extend target param must is object');
    }
    for (var i = 0; i < (arguments.length <= 1 ? 0 : arguments.length - 1); i++) {
      var source = i + 1 < 1 || arguments.length <= i + 1 ? undefined : arguments[i + 1];
      for (var name in source) {
        var src = target[name];
        var copy = source[name];

        //避免无限循环
        if (target === copy) {
          continue;
        }

        // 非可枚举属性
        if (!hasOwnProp(source, name)) {
          continue;
        }
        var copyIsArr = void 0;
        if (copy && (isObject(copy) || (copyIsArr = isArray(copy)))) {
          var clone = void 0;
          if (copyIsArr) {
            clone = src && isArray(src) ? src : [];
          } else {
            clone = src && isObject(src) ? src : {};
          }
          target[name] = extendDeep(clone, copy);
        } else if (typeof copy !== 'undefined') {
          target[name] = copy;
        }
      }
    }
    return target;
  }

  /*!
   * @templatejs/parser 3.1.3-alpha.0 (https://github.com/yanhaijing/template.js)
   * API https://github.com/yanhaijing/template.js
   * Copyright 2017-2023 templatejs. All Rights Reserved
   * Licensed under MIT (https://github.com/yanhaijing/template.js/blob/master/LICENSE)
   */


  /* eslint-disable no-var */
  var defaultOpt = {
    sTag: '<%',
    eTag: '%>',
    escape: true
  };
  function parsehtml(line) {
    // 单双引号转义
    line = String(line).replace(/('|")/g, '\\$1');
    var lineList = line.split(/\r\n|\n/);
    var code = '';
    for (var i = 0; i < lineList.length; i++) {
      code += ';__code__ += ("' + lineList[i] + (i === lineList.length - 1 ? '")\n' : '\\n")\n');
    }
    return code;
  }
  function parsejs(line) {
    var escape = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
    line = String(line);
    escape = !!escape;
    //var reg = /^(:?)(.*?)=(.*)$/;
    var reg = /^(?:=|(:.*?)=)(.*)$/;
    var html;
    var arr;
    var modifier;
    // = := :*=
    // :h=123 [':h=123', 'h', '123']
    if (arr = reg.exec(line)) {
      html = arr[2]; // 输出
      if (arr[1]) {
        // :开头
        modifier = arr[1].slice(1);
      } else {
        // = 开头
        modifier = escape ? 'h' : '';
      }
      return ';__code__ += __modifierMap__["' + modifier + '"](typeof (' + html + ') !== "undefined" ? (' + html + ') : "")\n';
    }
    //原生js
    return ';' + line + '\n';
  }
  function parse(tpl) {
    var opt = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultOpt;
    var _extendDeep = extendDeep({}, defaultOpt, opt),
      sTag = _extendDeep.sTag,
      eTag = _extendDeep.eTag,
      escape = _extendDeep.escape;
    tpl = String(tpl);
    var code = '';
    var tokens = tpl.split(sTag);
    for (var i = 0, len = tokens.length; i < len; i++) {
      var token = tokens[i].split(eTag);
      if (token.length === 1) {
        // html
        // <div></div>
        code += parsehtml(token[0]);
      } else {
        // js
        // <%= a%>
        code += parsejs(token[0], escape);
        if (token[1]) {
          // js + html
          // <%if () {%> html <%}%>
          code += parsehtml(token[1]);
        }
      }
    }
    return code;
  }

  /*!
   * @templatejs/runtime 3.1.3-alpha.0 (https://github.com/yanhaijing/template.js)
   * API https://github.com/yanhaijing/template.js/tree/master/packages/runtime/doc/api.md
   * Copyright 2017-2023 yanhaijing. All Rights Reserved
   * Licensed under MIT (https://github.com/yanhaijing/template.js/blob/master/LICENSE)
   */

  var o$1 = {
    sTag: '<%',
    //开始标签
    eTag: '%>',
    //结束标签
    compress: false,
    //是否压缩html
    escape: true,
    //默认输出是否进行HTML转义
    error: function error() {} //错误回调
  };

  function clone$1() {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }
    return extend.apply(null, [{}].concat(args));
  }
  function nothing(param) {
    return param;
  }
  function encodeHTML$1(source) {
    return String(source).replace(/&/g, '&amp;').replace(/</g, '&lt;').replace(/>/g, '&gt;').replace(/\\/g, '&#92;').replace(/"/g, '&quot;').replace(/'/g, '&#39;');
  }
  var functionMap$1 = {}; //内部函数对象
  //修饰器前缀
  var modifierMap$1 = {
    '': function _(param) {
      return nothing(param);
    },
    h: function h(param) {
      return encodeHTML$1(param);
    },
    u: function u(param) {
      return encodeURI(param);
    }
  };
  function consoleAdapter(cmd, msg) {
    typeof console !== 'undefined' && console[cmd] && console[cmd](msg);
  }
  function runtime() {}
  runtime.config = function (option) {
    if (type(option) === 'object') {
      o$1 = extend(o$1, option);
    }
    return clone$1(o$1);
  };
  runtime.compress = function (html) {
    return String(html).replace(/\s+/g, ' ').replace(/<!--[\w\W]*?-->/g, '');
  };
  runtime.handelError = function handelError(e) {
    var _o$error, _o;
    var message = 'template.js error\n\n';
    for (var key in e) {
      message += '<' + key + '>\n' + e[key] + '\n\n';
    }
    message += '<message>\n' + e.message + '\n\n';
    consoleAdapter('error', message);
    (_o$error = (_o = o$1).error) === null || _o$error === void 0 || _o$error.call(_o, e);
    function error() {
      return 'template.js error';
    }
    error.toString = function () {
      return '__code__ = "template.js error"';
    };
    return error;
  };
  runtime.registerFunction = function (name, fn) {
    if (typeof name !== 'string') {
      return clone$1(functionMap$1);
    }
    if (type(fn) !== 'function') {
      return functionMap$1[name];
    }
    return functionMap$1[name] = fn;
  };
  runtime.unregisterFunction = function (name) {
    if (typeof name !== 'string') {
      return false;
    }
    delete functionMap$1[name];
    return true;
  };
  runtime.registerModifier = function (name, fn) {
    if (typeof name !== 'string') {
      return clone$1(modifierMap$1);
    }
    if (type(fn) !== 'function') {
      return modifierMap$1[name];
    }
    return modifierMap$1[name] = fn;
  };
  runtime.unregisterModifier = function (name) {
    if (typeof name !== 'string') {
      return false;
    }
    delete modifierMap$1[name];
    return true;
  };
  runtime.encodeHTML = encodeHTML$1;
  runtime.functionMap = functionMap$1;
  runtime.modifierMap = modifierMap$1;
  runtime.o = o$1;

  /* eslint-disable @typescript-eslint/ban-types */
  /* eslint-disable no-var */
  /* eslint-disable prefer-spread */
  /* eslint-disable @typescript-eslint/no-explicit-any */
  var encodeHTML = runtime.encodeHTML,
    compress = runtime.compress,
    handelError = runtime.handelError,
    o = runtime.o,
    functionMap = runtime.functionMap,
    modifierMap = runtime.modifierMap;
  function clone() {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }
    return extend.apply(null, [{}].concat(args));
  }
  function compiler(tpl) {
    var opt = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : o;
    var mainCode = parse(tpl, opt);
    var headerCode = '\n' + '    var html = (function (__data__, __modifierMap__) {\n' + '        var __str__ = "", __code__ = "";\n' + '        for(var key in __data__) {\n' + '            __str__+=("var " + key + "=__data__[\'" + key + "\'];");\n' + '        }\n' + '        eval(__str__);\n\n';
    var footerCode = '\n' + '        ;return __code__;\n' + '    }(__data__, __modifierMap__));\n' + '    return html;\n';
    var code = headerCode + mainCode + footerCode;
    code = code.replace(/[\r]/g, ' '); // ie 7 8 会报错，不知道为什么
    try {
      var Render = new Function('__data__', '__modifierMap__', code);
      Render.toString = function () {
        return mainCode;
      };
      return Render;
    } catch (e) {
      e.temp = 'function anonymous(__data__, __modifierMap__) {' + code + '}';
      throw e;
    }
  }
  function compile(tpl) {
    var opt = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : o;
    opt = clone(o, opt);
    try {
      var Render = compiler(tpl, opt);
    } catch (e) {
      e.name = 'CompileError';
      e.tpl = tpl;
      e.render = e.temp;
      delete e.temp;
      return handelError(e);
    }
    function render(data) {
      data = clone(functionMap, data);
      try {
        var html = Render(data, modifierMap);
        html = opt.compress ? compress(html) : html;
        return html;
      } catch (e) {
        e.name = 'RenderError';
        e.tpl = tpl;
        e.render = Render.toString();
        return handelError(e)();
      }
    }
    render.toString = function () {
      return Render.toString();
    };
    return render;
  }
  function template(tpl, data) {
    if (typeof tpl !== 'string') {
      return '';
    }
    var fn = compile(tpl);
    if (type(data) !== 'object') {
      return fn;
    }
    return fn(data);
  }
  template.config = function (option) {
    return runtime.config(option);
  };
  template.registerFunction = function (name, fn) {
    return runtime.registerFunction(name, fn);
  };
  template.unregisterFunction = function (name) {
    return runtime.unregisterFunction(name);
  };
  template.registerModifier = function (name, fn) {
    return runtime.registerModifier(name, fn);
  };
  template.unregisterModifier = function (name) {
    return runtime.unregisterModifier(name);
  };
  // 兼容runtime, 预编译插件可以引用runtime，也可以引用template
  template.encodeHTML = encodeHTML;
  template.compress = compress;
  template.handelError = handelError;
  template.functionMap = functionMap;
  template.modifierMap = modifierMap;
  // 兼容旧版本
  template.__encodeHTML = encodeHTML;
  template.__compress = compress;
  template.__handelError = handelError;
  template.__compile = compile;

  return template;

}));
